/////////////////////////////////////////////////////////////////
// theseus.h

#ifndef MY_THESEUS
#define MY_THESEUS

#include "ct_windows.h"
#include "ct_graphics.h"
#include "ct_mouse.h"
#include "ct_delay.h"

#include "labyrinth.h"

// Stack of Point to keep the thread, implemented as a singly linked list.
// Instead of Point we could take any struct that contains a pair of ints.

struct PointNode {
  Point       data;
  PointNode * next;
  
  PointNode( Point d, PointNode * n ) : data(d), next(n) {}
};

class PointStack {
  PointNode * first;
public:
  PointStack() : first(NULL) {} // initially empty
  
  void Push( Point p ){
    PointNode * ptr = new PointNode( p, first );
    first = ptr;
  }

  void Pop(){
    if( first == NULL ) 
      return;           // nothing to pop
    // else there is at least one node
    PointNode * ptr = first;
    first = first->next;
    delete ptr;        // remove the popped node
  }

  Point Top(){  // return the top Point on the stack
    if( first == NULL ){
      cerr << "PointStack underflow\n";
      exit(-1);
    }
    // else the node on top is valid
    return first->data;
  }

  bool isEmpty(){
    return first == NULL;
  }

  ~PointStack(){  // we need to deallocate all nodes one by one
    PointNode * ptr = first;
    PointNode * doomed;
    while( ptr != NULL ){
      doomed = ptr;
      ptr = ptr->next;
      delete doomed;
    }
  }
};

/////////////////////////////////////////////////////////////////

class Theseus {
  int x;  // current coords
  int y; 
public:
  Theseus( ) : x(0), y(0) {}
  
  // solve the labytinth!
  void Solve( Labyrinth& lbr, int x_ = 0, int y_ = 0 ){

	x = x_;
	y = y_;

    PointStack st;  // create the stack to keep the path
    st.Push( MakePoint(x,y) );

    // Moves are tried in this order:
    //   East:  x+1, y
    //   South: x,   y+1
    //   West:  x-1, y
    //   North: x, y-1
    // The first direction in which there are free (and unmarked 
    // with thread) cells is tried.

    while( ! lbr.Contains(x,y,GOAL) && ! st.isEmpty() ){

	  lbr.Put(x,y,THREAD);      // mark current position
		
	  if( lbr.isFree( x+1, y ) ){
		st.Push( MakePoint(x,y) ); // push current position
		x = x+1;
      }
      else if( lbr.isFree( x, y+1 ) ){
		st.Push( MakePoint(x,y) ); // push current position
		y = y+1;
      }
      else if( lbr.isFree( x-1, y ) ){
		st.Push( MakePoint(x,y) ); // push current position
		x = x-1;
      }
      else if( lbr.isFree( x, y-1 ) ){
		st.Push( MakePoint(x,y) ); // push current position
		y = y-1;
      }
      else { // no move, back up one step
		Point p;
		p = st.Top();
		st.Pop();
		x = p.h;
		y = p.v;
      }
      lbr.Draw(x,y,THESEUS);
      Delay( 50 );
      // DEBUG:
      //Point dummy;
      //GetClickPoint(dummy);

    } // end while
    
    // why we exited?
    if( st.isEmpty() )
      cout << "No way! Not fair!\n";
    else
      cout << "Found it! Hurray!\n";

  }
};

#endif //  MY_THESEUS
